/*
 *Licensed Materials - Use restricted, please refer to the "Samples Gallery" terms and conditions in the IBM International Program License Agreement.
 *
 * Copyright IBM Corporation 2003, 2004. All Rights Reserved. 
 */
package superscript;

import com.rational.test.ft.BadArgumentException;
import com.rational.test.ft.object.interfaces.IWindow;
import com.rational.test.ft.object.interfaces.TestObject;
import com.rational.test.ft.vp.ITestDataTreeNode;
import com.rational.test.ft.vp.VpUtil;

/**
 * This class provides some methods that may be useful when testing 
 * SWT-based applications.  Note that this implementation makes use of WindowScript,
 * which is Microsoft-Windows specific.  This class will not work on Linux.
 */
public abstract class SwtScript extends WindowScript 
{
	private static final int ID_BUTTON_DEFINE_CUSTOM_COLORS = 0x2cf;
	private static final int ID_BUTTON_OK = 1;
	private static final int ID_EDIT_RED = 0x2c2;
	private static final int ID_EDIT_GREEN = 0x2c3;
	private static final int ID_EDIT_BLUE = 0x2c4;
	/**
	 * pick a color from a standard Color dialog. The dialog must be the
	 * top-level, active window (this method will wait for it to appear).
	 */
	protected void chooseColor(int r, int g, int b)
	{
		IWindow dlg = findActiveDialog();
		IWindow defineCustomColorsButton = childWindowFromId(dlg,ID_BUTTON_DEFINE_CUSTOM_COLORS);
		defineCustomColorsButton.click();
		
		childWindowFromId(dlg,ID_EDIT_RED).click();
		dlg.inputKeys("{end}+{home}"+r);
		
		childWindowFromId(dlg,ID_EDIT_GREEN).click();
		dlg.inputKeys("{end}+{home}"+g);
		
		childWindowFromId(dlg,ID_EDIT_BLUE).click();
		dlg.inputKeys("{end}+{home}"+b);
		
		childWindowFromId(dlg,ID_BUTTON_OK).click();
	}

	/**
	 * Perform a Menu Verification point on a SWT top-level menu. 
	 * This uses vpManual, so the first time you run the script, the baseline
	 * will be saved.
	 */
	protected void menuVP(String vpName, TestObject menu)
	{
		ITestDataTreeNode[] treeNodes = getMenuNodes(menu);		
		vpManual( vpName, VpUtil.getTestData(treeNodes)).performTest();
	}

	/**
	 * Get the menu nodes from a top-level swt menu
	 */
	private ITestDataTreeNode[] getMenuNodes(TestObject menu)
	{
		TestObject[] items = null;
		try
		{
			items  = (TestObject[])menu.invoke("getItems");
		}
		catch (Exception e)
		{
			throw new BadArgumentException("Error getting menu items - probably not a menu: "+
				menu);
		}
		
		int n = (items == null) ? 0 : items.length;
		if (n == 0)
			return null;
		
		// We have successfully found menu items.  Build the tree nodes
		// containing just the text.	
		ITestDataTreeNode[] treeNodes = new ITestDataTreeNode[n];

		for (int i = 0; i < n; ++i)
		{
			// for each item, get the text and use it to make the tree node.
			TestObject item = items[i];
			String menuText = (String)item.getProperty("text");
			treeNodes[i] = VpUtil.getTestDataTreeNode(menuText);
			
			// see if the item is itself a submenu
			TestObject subMenu = (TestObject)item.invoke("getMenu");
			if (subMenu != null)
			{
				// We have a submenu!  Recursively get its children nodes and
				// set the children	on this node. 
				ITestDataTreeNode[] subTreeNodes = getMenuNodes(subMenu );
				treeNodes[i].setChildren(subTreeNodes);
				
				subMenu.unregister(); // unregister the TestObject
			}
		}
		unregister(items);// unregister the aray of TestObjects
		return treeNodes;
	}


	/**
	 * pick a directory from a standard Directory dialog. The dialog must be the
	 * top-level, active window (this method will wait for it to appear).
	 */
	protected void chooseDirectory(String directory)
	{
		findActiveDialog().inputKeys(directory+"~");
	}

	
	/**
	 * pick a file from a standard File dialog. The dialog must be the
	 * top-level, active window (this method will wait for it to appear).
	 */
	protected void chooseFile(String file)
	{
		findActiveDialog().inputKeys(file+"~");
	}
	
	/**
	 * General-purpose exception for swt scripts
	 */
	public static class SwtScriptException extends RuntimeException
	{
		SwtScriptException(String s) { super(s); }
	}
	
	
}

